#include <iostream>
#include <cstdio>
#include <cstdlib>
#include "cholesky.h"
#include "wallclocktime.h"

void fill_spd_matrix(int n, double *A);

int main(int argc, char **argv)
{
   int n=200;
   if(argc>1)
      std::sscanf(argv[1], "%d", &n);
   else
      std::cout<<"You can specify the matrix size (default="<<n<<") as an argument"<<std::endl;
   std::cout<<"Testing "<<n<<"x"<<n<<" matrix"<<std::endl;
   double *A=(double*)std::malloc(n*n*sizeof(double));

   // first: time filling the matrix ========================================

   // warm up the system
   fill_spd_matrix(n, A);
   // start timing
   set_time_base();
   double start_time=get_time_in_seconds(), end_time;
   int iteration=0;
   do{
     fill_spd_matrix(n, A);
     ++iteration;
     end_time=get_time_in_seconds();
   }while(iteration<3 || end_time-start_time<0.5);
   double fill_time=(end_time-start_time)/iteration;
   std::cout<<"Fill time is "<<fill_time<<" measured over "<<iteration<<" iterations"<<std::endl;

   // next: time cholesky_ijk ========================================

   // warm up the system
   fill_spd_matrix(n, A);
   cholesky_ijk(n, A);
   // start timing
   start_time=get_time_in_seconds();
   iteration=0;
   do{
     fill_spd_matrix(n, A);
     cholesky_ijk(n, A);
     ++iteration;
     end_time=get_time_in_seconds();
   }while(iteration<3 || end_time-start_time<1);
   double ijk_time=(end_time-start_time)/iteration-fill_time;
   std::cout<<"ijk time is "<<ijk_time<<" measured over "<<iteration<<" iterations"<<std::endl;

   // next: time cholesky_kij ========================================

   // warm up the system
   fill_spd_matrix(n, A);
   cholesky_kij(n, A);
   // start timing
   start_time=get_time_in_seconds();
   iteration=0;
   do{
     fill_spd_matrix(n, A);
     cholesky_kij(n, A);
     ++iteration;
     end_time=get_time_in_seconds();
   }while(iteration<3 || end_time-start_time<1);
   double kij_time=(end_time-start_time)/iteration-fill_time;
   std::cout<<"kij time is "<<kij_time<<" measured over "<<iteration<<" iterations"<<std::endl;

   // next: time cholesky_LAPACK  ========================================

   // warm up the system
   fill_spd_matrix(n, A);
   cholesky_LAPACK(n, A);
   // start timing
   start_time=get_time_in_seconds();
   iteration=0;
   do{
     fill_spd_matrix(n, A);
     cholesky_LAPACK(n, A);
     ++iteration;
     end_time=get_time_in_seconds();
   }while(iteration<3 || end_time-start_time<1);
   double lapack_time=(end_time-start_time)/iteration-fill_time;
   std::cout<<"LAPACK time is "<<lapack_time<<" measured over "<<iteration<<" iterations"<<std::endl;

   std::free(A);
   return 0;
}

void fill_spd_matrix(int n, double *A)
{
   for(int j=0; j<n; ++j) for(int i=0; i<n; ++i){
      A[i+n*j]=-1;
   }
   for(int i=0; i<n; ++i)
      A[i+n*i]=2*n; // safely SPD
}

