#include <cmath>
#include <cstdio>
#include <fstream>
#include "gluvi.h"
#include "util.h"

int nmesh=0;
double xmin, xmax, dx, ymin, ymax, dy;
double *fmesh=0;
double f_min, f_max;
int n;
double *xy=0, *f=0;

void color_point(float f)
{
   double s=3*(f-f_min)/(f_max-f_min);
   if(s<1)
      glColor3f(1-s, s, 0);
   else if(s<2)
      glColor3f(0, 2-s, s-1);
   else
      glColor3f(s-2, 0, 3-s);
}

void display(void)
{
   glDisable(GL_LIGHTING);
   for(int j=1; j<nmesh; ++j){
      glBegin(GL_QUAD_STRIP);
      for(int i=0; i<nmesh; ++i){
         color_point(fmesh[i+nmesh*(j-1)]);
         glVertex3d(xmin+dx*i, fmesh[i+nmesh*(j-1)], ymin+dy*(j-1));
         color_point(fmesh[i+nmesh*j]);
         glVertex3d(xmin+dx*i, fmesh[i+nmesh*j], ymin+dy*j);
      }
      glEnd();
   }
   glColor3f(0, 0, 0);
   glPointSize(5);
   glBegin(GL_POINTS);
   for(int i=0; i<n; ++i)
      glVertex3d(xy[2*i], f[i], xy[2*i+1]);
   glEnd();
}

void read_mesh(const char *filename)
{
   std::ifstream input(filename);
   input>>nmesh;
   input>>xmin>>xmax>>ymin>>ymax;
   dx=(xmax-xmin)/(nmesh-1);
   dy=(ymax-ymin)/(nmesh-1);
   fmesh=new double[nmesh*nmesh];
   f_min=MAXFLOAT;
   f_max=-MAXFLOAT;
   for(int j=0; j<nmesh; ++j) for(int i=0; i<nmesh; ++i){
      input>>fmesh[i+nmesh*j];
      if(fmesh[i+nmesh*j]<f_min) f_min=fmesh[i+nmesh*j];
      if(fmesh[i+nmesh*j]>f_max) f_max=fmesh[i+nmesh*j];
   }
}

void read_points(const char *filename)
{
   std::ifstream input(filename);
   input>>n;
   xy=new double[2*n];
   f=new double[n];
   for(int i=0; i<n; ++i)
      input>>xy[2*i]>>xy[2*i+1]>>f[i];
}

void set_camera(Gluvi::Target3D &cam)
{
   if(nmesh==0)
      return;
   float zmin=MAXFLOAT, zmax=-MAXFLOAT;
   for(int i=0; i<nmesh*nmesh; ++i){
      if(fmesh[i]<zmin) zmin=fmesh[i];
      if(fmesh[i]>zmax) zmax=fmesh[i];
   }
   cam.target[0]=0.5*(xmin+xmax);
   cam.target[1]=0.5*(ymin+ymax);
   cam.target[2]=0.5*(zmin+zmax);
   cam.dist=sqrt(sqr(xmax-xmin)+sqr(ymax-ymin)+sqr(zmax-zmin));
}

struct ScreenShotButton : public Gluvi::Button{
   const char *filename;
   ScreenShotButton(const char *label, const char *filename_)
      : Gluvi::Button(label), filename(filename_)
   {}

   void action()
   { Gluvi::ppm_screenshot("%s", filename); }
};

int main(int argc, char **argv)
{
   Gluvi::init(argv[0], &argc, argv);
   Gluvi::Target3D cam;
   Gluvi::camera=&cam;

   glClearColor(1,1,1,0);
   glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, 1);

   Gluvi::userDisplayFunc=display;
   ScreenShotButton screenshot("Screenshot", "rbf.ppm");
   Gluvi::root.list.push_back(&screenshot);

   if(argc>1)
      read_mesh(argv[1]);
   if(argc>2)
      read_points(argv[2]);

   set_camera(cam);

   Gluvi::run();
   return 0;
}

